/*----------------------------------------------------------
*                                      
*    __  __                      
*   | | / /__  __ _____    ___    __   _____  ___
*   |  / / \ \/ //     \  /   \ /`__'\|  __| /   \
*   |  \ \  \  // /| |\ \|  O  |  ___/| |  `|  O  | 
*   |_| \_\  |_|_/ |_| \_\\_/|_|\___\ |_|    \_/|_|
*                  
*
*   Kymaera (c) 2004 
*	Universal Arcade Cabinet Front End
*
*	Steve Hernandez: steveh@crosslink.net 
*   Windows, Linux and Dos version
*   See the Readme.txt and License.txt for more information          
*
*
* -----------------------------------------------------------
*/

#include "listitem.h"



//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


List::List(){
	list =0;
	lastitem=0;
	numelements = 0;
}
List::~List(){Destroy();}

bool List::MoveItemUp(void *item){
    Listitem  *curr=0, *node=0;
	void *prevdata;
	void *thisdata;

    int  inlist = false;

    for(curr= list; curr!=0; curr=curr->next) {
        if(curr->data == item) {
            inlist = true;
            node = curr;
            break;
        }
    }
	if(!inlist)return false;
	if(!node->prev)return false ;// handle case 2, first in list already
	// I think the easiest way is to swap the data pointer
	prevdata = node->prev->data;
	thisdata = node->data;
	node->prev->data = thisdata;
	node->data = prevdata;
	return true;
}
bool List::MoveItemDown(void *item){
    Listitem  *curr, *node=0;
	void *nextdata;
	void *thisdata;

    int  inlist = false;

    for(curr= list; curr!=0; curr=curr->next) {
        if(curr->data == item) {
            inlist = true;
            node = curr;
            break;
        }
    }
	if(!inlist)return false;
	if(!node->next)return false ;// handle case 2, first in list already
	// I think the easiest way is to swap the data pointer
	nextdata = node->next->data;
	thisdata = node->data;
	node->next->data = thisdata;
	node->data = nextdata;
	return true;

}

bool List::Add(void *item){
        Listitem        *newnode;//, *curr;

        if((newnode = new Listitem()) == 0) {
//                Log("AddItem: Error allocating new list node\n");
                return false;
        }
	numelements++;
        if(list == 0) {
                /* Must construct a new list
                 * So set this to be the head, write it, and return true
                 */

                newnode->data = item;
                list = newnode;
				lastitem =newnode; // set the last node
                return true;
        } else {
                /* Find the end of the list
                 * Check to see if we find the item we're about to insert
                 * If so, return false, because we don't want duplicate
                 * items in the list
                 */

			newnode->data = item;
			lastitem->next = newnode;
			newnode->next = 0;
			newnode->prev =lastitem;
			//now set the new lastitem
			lastitem = newnode;
			/*
                for(curr = list; curr->next!=0; curr=curr->next) {
                        if(curr->data == item)
                                return false;
                }

                newnode->data = item;
                curr->next = newnode;
                newnode->next = 0;
                newnode->prev = curr;
				*/
				
                return true;
        }
}
bool List::InList(void *item){//check to see if this item is already in the list
	bool retval = false;
		Listitem        *curr;
        for(curr= list; curr!=0; curr=curr->next) {
                if(curr->data == item) {
                        retval = true;
                        break;
                }
        }
	return retval;
}
bool List::Remove(int idx){
        Listitem        *curr=0, *node=0;
        int             inlist = false;
		int lidx =0;

        /* Search through the list, to see if we can find the item we wish
         * to remove. If its not there, it can't be removed from the list
         * therefore we signal an error
         */

        for(curr= list; curr!=0; curr=curr->next,lidx++) {
			/*
                if(curr->data == item) {
                        inlist = true;
                        node = curr;
                        break;
                }
				*/
				if(lidx == idx){
                        inlist = true;
                        node = curr;
                        break;				
				}
        }

        if(!inlist) {
         //       printf("RemoveItem: Warning, item not found in list\n");
                return false;
        }

        /* Head of the list is a special case, we must deal with it
         * seperately
         */
	numelements--;
        if(node == list) {
                list = node->next;
				if(list != 0){
                    (list)->prev = 0;
				}else{
					lastitem =0;
				}
				delete node;
                return true;
        } else if (node == lastitem){ // last item
			node->prev->next = 0;
			lastitem = node->prev;
			delete node;
			return true;

		}else {
                if(node->prev != 0)
                        node->prev->next = node->next;
                if(node->next != 0)
                        node->next->prev = node->prev;
				delete node;
                return true;
        }
}
bool List::Remove(void *item)
{
        Listitem        *curr=0, *node=0;
        int             inlist = false;

        /* Search through the list, to see if we can find the item we wish
         * to remove. If its not there, it can't be removed from the list
         * therefore we signal an error
         */

        for(curr= list; curr!=0; curr=curr->next) {
                if(curr->data == item) {
                        inlist = true;
                        node = curr;
                        break;
                }
        }

        if(!inlist) {
         //       printf("RemoveItem: Warning, item not found in list\n");
                return false;
        }

        /* Head of the list is a special case, we must deal with it
         * seperately
         */
	numelements--;
        if(node == list) {
                list = node->next;
				if(list != 0){
                    (list)->prev = 0;
				}else{
					//new version
					lastitem =0;
				}
				delete node;
                return true;
        } else if (node == lastitem){ // last item
			node->prev->next = 0;
			lastitem = node->prev;
			delete node;
			return true;

		}else {
                if(node->prev != 0)
                        node->prev->next = node->next;
                if(node->next != 0)
                        node->next->prev = node->prev;
				delete node;
                return true;
        }
}

/*
 * Function: DestroyList
 * Purpose : Release all the memory allocated to the nodes in the list
 * Input   : List
 * Output  : Nothing returned
 */

void List::Destroy(){
        Listitem        *node, *next;
        for(node= list; node!=0; node=next) {
                next = node->next;
                //free(node);
				delete node;
        }
		list = 0;//set the head of the list to be 0
		numelements = 0;
		lastitem=0;
}

/*
 * Function: CountItems
 * Purpose : Count the number of items in the list
 * Input   : The list
 * Output  : Number of items
 */

int List::Count(){
//        Listitem        *node;
    //    int             count = 0;
  //      for(node= list; node!=0; node=node->next)
      //          count++;

        return (int)numelements;//count;
}


Listitem * List::GetListitem(int index){
        Listitem        *node;

        for(node= list; node!=0; node=node->next) {
                if(index-- == 0)
                        return node;
        }
        return 0;
}

/*
 * Function: GetItem
 * Purpose : Get the item for a given index (zero-based)
 * Input   : List, index
 * Output  : Item pointer, 0 if not found
 */

void *List::GetItem(int index)
{
        Listitem        *node;

        /* Advance thru the list, checking to see if index == 0. We treat
         * index as a counter for "number of nodes to traverse to get to
         * the node we want to". So when its 0, its the head, when its 10,
         * its 10 nodes from the head, etc etc
         */

        for(node= list; node!=0; node=node->next) {
                if(index-- == 0)
                        return node->data;
        }

        return 0;
}

/*
 * Function: GetItemIndex
 * Purpose : Get an index, for an item. Return -1 on failure
 * Input   : Item pointer
 * Output  : Valid zero-based index if ok, else -1
 */

int List::GetItemIndex(void *item)
{
        Listitem        *node=0;
        int             index = 0;

        for(node= list; node!=0; node=node->next) {
			if(node->data == item){
                        return index;
			}
            index++;
        }

        return -1;
}
