/*----------------------------------------------------------
*                                      
*    __  __                      
*   | | / /__  __ _____    ___    __   _____  ___
*   |  / / \ \/ //     \  /   \ /`__'\|  __| /   \
*   |  \ \  \  // /| |\ \|  O  |  ___/| |  `|  O  | 
*   |_| \_\  |_|_/ |_| \_\\_/|_|\___\ |_|    \_/|_|
*                  
*
*   Kymaera (c) 2004 
*	Universal Arcade Cabinet Front End
*
*	Steve Hernandez: steveh@crosslink.net 
*   Windows, Linux and Dos version
*   See the Readme.txt and License.txt for more information          
*
*
* -----------------------------------------------------------
*/

#include "soungmanager.h"
#include "log.h"
#include "fileutil.h"
#include "memutil.h"
#include "time.h"
#include "porting.h"
#include "dirmanager.h"
#ifdef LINUX

/*
	,
	,
	SELECT,
	EXIT_GAME,
	PAGEUP,
	PAGEDOWN,
	NEXT_SONG,
	PREV_SONG,
	ADDTOFAV,
	EXIT_AND_SHUTDOWN,
	BACK
*/
soundentry soundmap[] = {
	{MOVE_DOWN,"sound/click.wav"},
	{MOVE_UP,"sound/click.wav"},
	//{NEXT_CATAGORY,"sound/click.wav"},
	//{PREV_CATAGORY,"sound/click.wav"},
	{SELECT,"sound/click.wav"},
	{EXIT_GAME,"sound/click.wav"},
	{PAGEUP,"sound/click.wav"},
	{PAGEDOWN,"sound/click.wav"},
	//{NEXT_EMULATOR,"sound/click.wav"},
	//{PREV_EMULATOR,"sound/click.wav"},
	{NEXT_SONG,"sound/click.wav"},
	{PREV_SONG,"sound/click.wav"},
	{ADDTOFAV,"sound/click.wav"}

};

#else

soundentry soundmap[] = {
	{MOVE_DOWN,"sound\\click.wav"},
	{MOVE_UP,"sound\\click.wav"},
	//{NEXT_CATAGORY,"sound\\click.wav"},
	//{PREV_CATAGORY,"sound\\click.wav"},
	{SELECT,"sound\\click.wav"},
	{EXIT_GAME,"sound\\click.wav"},
	{PAGEUP,"sound\\click.wav"},
	{PAGEDOWN,"sound\\click.wav"},
	//{NEXT_EMULATOR,"sound\\click.wav"},
	//{PREV_EMULATOR,"sound\\click.wav"},
	{NEXT_SONG,"sound\\click.wav"},
	{PREV_SONG,"sound\\click.wav"},
	{ADDTOFAV,"sound\\click.wav"}

};

#endif

void soundentry::Save(CMarkupSTL *xmldoc){
	SaveXMLString(xmldoc,"SoundFile",soundfile);
	SaveXMLVar(xmldoc,"EventCode",(int)event);
}
void soundentry::Load(CMarkupSTL *xmldoc){
	int ev;
	string str;
	LoadXMLString(xmldoc,"SoundFile",&str);
	LoadXMLVar(xmldoc,"EventCode",&ev);
	event = (EventCodes)ev;
	strcpy(soundfile,str.c_str());

}

void SoundManager::Init()
{
	strm1 = new SoundFile();
	strm2 = new SoundFile();
	m_mp3random = true;
	m_playmp3 = true;
   srand( (unsigned)time( NULL ) );
   m_SoundMapFile = "config"+pathsep+"soundmap.smp";
	CreateDefault();
}

SoundManager::SoundManager(void)
{
	strm1 = new SoundFile();
	strm2 = new SoundFile();
	m_mp3random = true;
	m_playmp3 = true;
   srand( (unsigned)time( NULL ) );
   m_SoundMapFile = "config"+pathsep+"soundmap.smp";
	CreateDefault();
}

void SoundManager::Release(){
	Listitem *li;
	soundentry *se;
	for (li = m_SoundEntries.list;li !=0;li=li->next){
		se = (soundentry *)li->data;
		delete se;
	}		
	m_SoundEntries.Destroy();
	ReleaseMP3PlayList();
	/*
	if(strm1){
		strm1->Stop();
		delete strm1;
		strm1=0;
	}
	if(strm2){
		strm2->Stop();
		delete strm2;	
		strm2=0;
	}
	*/
}

SoundManager::~SoundManager(void)
{
	Release();
	Shutdown();
}

void SoundManager::PlayEventSound(EventCodes evt){
	Listitem *li;
	soundentry *se;
	for (li = m_SoundEntries.list;li !=0;li=li->next){
		se = (soundentry *)li->data;
		if(se->event == evt){
			Play(se->soundfile,1);
			return;
		}
	}
}
void SoundManager::CreateDefault(){
	soundentry *se;
	int num = sizeof(soundmap)/sizeof(soundentry);
	for(int c = 0; c<num;c++){
		se = new soundentry;
		se->event = soundmap[c].event;
		strcpy(se->soundfile,soundmap[c].soundfile);
		m_SoundEntries.Add(se);
	}
}
bool SoundManager::Startup(){
	return true;
}
void SoundManager::Shutdown(){
	if(strm1){
		strm1->Stop();
		delete strm1;
		strm1 = 0;
	}
	if(strm2){
		strm2->Stop();
		delete strm2;
		strm2 = 0;
	}
}
void SoundManager::Pause(){
	if(strm1)
		strm1->Pause();
	if(strm2)
		strm2->Pause();
}
void SoundManager::Resume(){
	if(strm1)
		strm1->Resume();
	if(strm2)
		strm2->Resume();
}
void SoundManager::NextSong(){
	Listitem *li  = m_curMP3;
	string *str;
	li=li->next;
	if(!li){
		if(!m_Mp3PlayList.list ) return;
		li = m_Mp3PlayList.list;
	}
	m_curMP3 = li;
	str = (string *)li->data;
	string s;
	s = *str;
	Play((char *)s.c_str(),2);
}
void SoundManager::PreviousSong(){
	Listitem *li  = m_curMP3;
	string *str;
	li=li->prev;
	if(!li){
		if(!m_Mp3PlayList.list ) return;
		li = m_Mp3PlayList.lastitem ;
	}
	m_curMP3 = li;
	str = (string *)li->data;
	string s;
	s = *str;
	Play((char *)s.c_str(),2);
}

void SoundManager::Update(){
	string *mp3filename,str;
	char tmp[255];
	if(!strm1)
		strm1 = new SoundFile();
	if(!strm2)
		strm2 = new SoundFile();

	strm1->Update();
	strm2->Update();
	if(!strm2->IsPlaying() ){
		if(m_playmp3){//load the next one			
			if(m_curMP3){
				if(!m_mp3random){
					if(m_curMP3->next){
						m_curMP3 = m_curMP3->next; // next one
					}else{
						m_curMP3 = m_Mp3PlayList.list; // first
					}						
				}else{ // random
					if(!m_Mp3PlayList.list ) return;
					m_curMP3 = m_Mp3PlayList.GetListitem(rand()%m_Mp3PlayList.numelements);
				}
				if(m_curMP3){
					mp3filename = (string *)m_curMP3->data;
					str = *mp3filename;
					strcpy(tmp,str.c_str());
					Play(tmp,2);
				}//m_curMP3
			} //m_curMP3
		} // m_playmp3
	}
}

void  SoundManager::Play(char *filename,int channel){	
	DirMgr->RestoreCWD();
	switch(channel){
		case 1:
			strm1->Stop();
			strm1->Load(filename);
			strm1->Play();
			break;
		case 2:
			strm2->Stop();
			strm2->Load(filename);
			strm2->Play();
			break;
	}
}


void SoundManager::Load(string filename){

	DirMgr->RestoreCWD();

	CMarkupSTL xmldoc;
	string Name;
	char tmp[256];
	if(!xmldoc.Load(filename.c_str())){
		return;
	}
	m_SoundMapFile = filename;
	Release();
	xmldoc.FindElem("SoundMap");
	xmldoc.IntoElem();
	LoadXMLString(&xmldoc,"PlayListFileName",&m_Mp3PlayListFileName);
	LoadXMLBool(&xmldoc,"PlayMp3",&m_playmp3);
	LoadXMLBool(&xmldoc,"Random",&m_mp3random);
	xmldoc.FindElem("SoundMapEvents");
	xmldoc.IntoElem();
	int num =0;// m_SoundEntries.numelements ;
	LoadXMLVar(&xmldoc,"NumberEvents",&num);
	soundentry *se;
	int c=0;
	for (c=0;c<num;c++){
		sprintf(tmp,"Entry_%d",c);
		xmldoc.FindElem(tmp);
		xmldoc.IntoElem();
		se = new soundentry;
		se->Load(&xmldoc);
		m_SoundEntries.Add(se);
		xmldoc.OutOfElem(); //entry_x
	}
	xmldoc.OutOfElem(); //soundmapevents
	xmldoc.OutOfElem(); // soundmap
	LoadMP3PlayList(m_Mp3PlayListFileName);
	//load the sound schema
}
void SoundManager::Save(string filename){
	DirMgr->RestoreCWD();
	CMarkupSTL xmldoc;
	string Name;
	char tmp[256];
	xmldoc.AddElem("SoundMap");
	xmldoc.IntoElem();

	SaveXMLString(&xmldoc,"PlayListFileName",m_Mp3PlayListFileName);
	SaveXMLBool(&xmldoc,"PlayMp3",m_playmp3);
	SaveXMLBool(&xmldoc,"Random",m_mp3random);
	xmldoc.AddElem("SoundMapEvents");
	xmldoc.IntoElem();
	int num = m_SoundEntries.numelements ;
	SaveXMLVar(&xmldoc,"NumberEvents",num);

	soundentry *se;
	int c=0;
	for (Listitem *li = m_SoundEntries.list ; li!=0; li=li->next,c++ ){
		sprintf(tmp,"Entry_%d",c);
		xmldoc.AddElem(tmp);
		xmldoc.IntoElem();
		se = (soundentry *)li->data;
		se->Save(&xmldoc);
		xmldoc.OutOfElem(); //entry_x
	}
	xmldoc.OutOfElem(); //soundmapevents
	xmldoc.OutOfElem(); // soundmap
	xmldoc.Save(filename.c_str ());
}

void SoundManager::ReleaseMP3PlayList(){
	if(m_Mp3PlayList.list )
		DelStrList(m_Mp3PlayList);
}

void SoundManager::LoadMP3PlayList(string filename){
	ReleaseMP3PlayList();
	if(filename == "")return;
	FILE *fp = fopen(filename.c_str(),"rt");
	if(!fp)
		return;
	char tmp[256];
	string *str;
	int len;
	while(fgets(tmp,255,fp)){		
		len = (int)strlen(tmp);
		tmp[len-1]=0;
		str = new string(tmp);
		//str->Trim();
		m_Mp3PlayList.Add(str);
	}
	m_curMP3 = m_Mp3PlayList.list;
	m_Mp3PlayListFileName = filename;
	fclose(fp);
}
